#!/usr/bin/env python

import sys
import os
import re
import fnmatch

def safe_file_name(file):
    return os.path.basename(file).replace('-', '_')

def create_file_xml(file):
    return '<File Id="%s" Source="%s" />' % (safe_file_name(file), file)

def create_files_xml_for_dir(dir, pattern):
    list = []
    for f in os.listdir(dir):
        file = os.path.join(dir, f)
        if os.path.isfile(file) and fnmatch.fnmatch(file, pattern):
            list.append(create_file_xml(file))

    return "\n".join(list)

def subst_files(buffer, top_dir, module):
    filterdict = {}
    filterdict['lib'] = '*.lib'
    filterdict['pc'] = '*.pc'

    dict = {}
    dict['input_path'] = top_dir

    l = re.findall("%\((.*)_files\)s", buffer)
    for i in l:
        if i == 'pc':
            dir = os.path.join(top_dir, 'lib', 'pkgconfig')
        else:
            dir = os.path.join(top_dir, i)

        if i in filterdict:
            dict["%s_files" % i] = create_files_xml_for_dir(dir, filterdict[i])
        else:
            dict["%s_files" % i] = create_files_xml_for_dir(dir, '*')

    print buffer % dict
    return 0

def main(argv):
    binDir = os.path.abspath(os.path.dirname(sys.argv[0]))

    if len(argv) == 2 and (argv[1] == '--help' or argv[1] == '-h' or argv[1] == '/help' or argv[1] == '/?'):
        print "Usage:\n\t%s <template> <path> <module>" % os.path.basename(argv[0])
        return 0

    retValue = 1
    if (len(argv) == 4):
        path = os.path.abspath(argv[2])
        if not os.path.exists(argv[1]):
            print "Unable to find the template file: %s\n" % argv[1]
        elif not os.path.exists(path):
            print "Unable to find the path: %s\n" % argv[2]
        else:
            f = file(argv[1], "r")
            retValue = subst_files(f.read(), path, argv[3])

    return retValue

if (__name__ == "__main__"):
    sys.exit(main(sys.argv))
