/*
 * Copyright (C) 2008 Haakon Sporsheim <haakon@ieei.no>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#define _HAS_EXCEPTIONS 0
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define LINK_EXE_WRAPPED    "link_.exe"
#define UTF16_BE_BOM        0xFFFE
#define UTF16_LE_BOM        0xFEFF

bool is_white(char ch)    { return ch == 0 || ch ==  ' ' || ch ==  '\t' || ch ==  '\r' || ch ==  '\n'; }
bool is_white(wchar_t ch) { return ch == 0 || ch == L' ' || ch == L'\t' || ch == L'\r' || ch == L'\n'; }

void append_wide_string (const wchar_t * str, size_t size, std::string & out)
{
  int mbsize = WideCharToMultiByte(CP_UTF8, 0, str, size, NULL, 0, NULL, NULL);
  char * tmp = new char[mbsize+1];
  tmp[mbsize] = 0;
  WideCharToMultiByte(CP_UTF8, 0, str, size, tmp, mbsize, NULL, NULL);
  out += tmp;
  delete [] tmp;
}

const char * find_white (const char * buf)
{
  while (!is_white(*buf)) buf++;
  return *buf ? buf : NULL;
}

const wchar_t * find_white (const wchar_t * buf)
{
  while (!is_white(*buf)) buf++;
  return *buf ? buf : NULL;
}

void parse_and_strip_cmd_rsp_buf (const char * const buf, std::streamsize size, std::ostream & stream, std::string & out)
{
  const char * lp = buf;
  const char * pos = strchr(lp, '@');

  while (pos)
  {
    stream.write(lp, pos - lp);
    lp = pos;
    pos = (lp[1] == '"') ? strchr(&lp[2], '"') : find_white(&lp[1]);

    out += ' ';
    if (pos == NULL)
    {
      out += lp;
      return;
    }
    else
    {
      if (*pos == '"') pos++;
      out.append(lp, pos - lp);
      lp = pos;
    }

    pos = strchr(lp, '@');
  }

  stream.write(lp, size - (lp - buf));
}

void parse_and_strip_cmd_rsp_buf_utf16le (const wchar_t * const buf, std::streamsize size, std::ostream & stream, std::string & out)
{
  const wchar_t * lp = buf;
  const wchar_t * pos = wcschr(lp, L'@');

  while (pos)
  {
    stream.write((char*)lp, (char*)pos - (char*)lp);
    lp = pos;
    pos = (lp[1] == L'"') ? wcschr(&lp[2], L'"') : find_white(&lp[1]);

    out += ' ';
    if (pos == NULL)
    {
      append_wide_string(lp, -1, out);
      return;
    }
    else
    {
      if (*pos == L'"') pos++;
      append_wide_string(lp, pos - lp, out);
      lp = pos;
    }

    pos = wcschr(lp, L'@');
  }

  stream.write((char*)lp, size - ((char*)lp - (char*)buf));
}

void parse_and_strip_cmd_rsp_buf_utf16be (const wchar_t * const buf, std::streamsize size, std::ostream & stream, std::string & out)
{
  std::cout << "UTF16 BE Not supported" << std::endl;
  exit(-1);
}

bool parse_cmd_rsp_file (LPCSTR file, std::string & out, std::vector<std::string> & tmp_files)
{
  char tmp[MAX_PATH];
  if (*file == '"') file++;
  strcpy(tmp, file);

  int last = strlen(tmp);
  if (tmp[last] == '"') tmp[last] = 0;
  strcat(tmp, "~");

  out += '@';
  out += tmp;
  bool ret = true;
  {
    std::ifstream in_file(file, std::ios::in | std::ios::ate | std::ios::binary);
    if (!in_file.is_open())
      return false;

    tmp_files.push_back(tmp);
    std::ifstream::pos_type size = in_file.tellg();
    in_file.seekg(0, std::ios::beg);
    char * buf = new char[4+size];
    in_file.read(buf, size);
    memset(buf+size,0,4);
    in_file.close();

    std::ofstream tmp_file(tmp, std::ios::out | std::ios::trunc | std::ios::binary);
    if (tmp_file.is_open())
    {
      if (*((USHORT*)buf) == UTF16_LE_BOM)
        parse_and_strip_cmd_rsp_buf_utf16le ((wchar_t*)buf, size, tmp_file, out);
      else if (*((USHORT*)buf) == UTF16_BE_BOM)
        parse_and_strip_cmd_rsp_buf_utf16be ((wchar_t*)buf, size, tmp_file, out);
      else
        parse_and_strip_cmd_rsp_buf (buf, size, tmp_file, out);

      tmp_file.flush();
      tmp_file.close();
    }
    else
    {
      ret = false;
    }

    delete [] buf;
  }

  return ret;
}

int main (int argc, char ** argv)
{
  std::string cmdline(LINK_EXE_WRAPPED);
  std::vector<std::string> tmp_rsp_files;
  for (int i = 1; i < argc; i++)
  {
    cmdline += ' ';
    if (argv[i][0] == '@')
      parse_cmd_rsp_file(&argv[i][1], cmdline, tmp_rsp_files);
    else
      cmdline += argv[i];
  }

  PROCESS_INFORMATION pi = {0};
  STARTUPINFO si = {0};
  si.cb = sizeof (STARTUPINFO);

  char cur_dir[MAX_PATH];
  if (!GetCurrentDirectory(MAX_PATH, cur_dir))
    cur_dir[0] = 0;

  std::cout << cmdline << std::endl;

  // Run MS link.exe
  if (!CreateProcess(NULL, (LPSTR)cmdline.c_str(), NULL, NULL, TRUE, 0, NULL, cur_dir[0] ? cur_dir : NULL, &si, &pi))
  {
    DWORD ret = GetLastError();
    std::cout << "Create process failed: " << ret << std::endl;
    return ret;
  }

  ::WaitForSingleObject(pi.hProcess, INFINITE);

  for (std::vector<std::string>::const_iterator it = tmp_rsp_files.begin();
      it != tmp_rsp_files.end(); ++it)
  {
    DeleteFile(it->c_str());
  }

  DWORD ret = -1;
  GetExitCodeProcess(pi.hProcess, &ret);
  return ret;
}
